/*==============================================================================*/
/*  Copyright (C) 2009-2020, Heartland Data inc. All Rights Reserved.           */
/*                                                                              */
/*  Title  :   Data Writing Driver                                              */
/*  EventID:   Via the driver                                                   */
/*  AppVer :   DT10 Ver10.XX`                                                  */
/*  FileID :   38b                                                              */
/*  Version:   2.2                                                              */
/*  Author :   HLDC                                                             */
/*==============================================================================*/

/*==============================================================================*/
/*  Desc:   Header for File IO                                                  */
/*==============================================================================*/
#include <stdio.h>
#include <sys/types.h>
#include <sys/time.h>
#include <time.h>
#include <semaphore.h>

/*==============================================================================*/
/*  Macro:  DT_UINT                                                             */
/*  Desc:   Please change Test Point argument type for DT10 Project setting.    */
/*==============================================================================*/
#define	DT_UINT unsigned int

/*==============================================================================*/
/*  Desc:   Test Point Parametaer                                               */
/*==============================================================================*/
#define DT_VARIABLE_BIT		0x2

/*==============================================================================*/
/*  Macro:  DT_ADD_EVENT_INFO                                                   */
/*  Desc:   Please set 1 when you add Event Value to Test Point.                */
/*==============================================================================*/
#define	DT_ADD_EVENT_INFO	1

static FILE *dat_file = NULL;
static int BusoutMode = 0;
unsigned int pid;
unsigned int cpu_id = 0;

/*==============================================================================*/
/*  Desc:   semaphore                                                           */
/*==============================================================================*/
#define DT_USE_CRITICALSECTION	0
sem_t semid;

#define SEM_INIT	sem_init(&semid, 0, 1)
#define SEM_WAIT	sem_wait(&semid)
#define SEM_POST	sem_post(&semid)
#define SEM_DESTROY	sem_destroy(&semid)

/*==============================================================================*/
/*  Macro:  BusOutModeSet                                                       */
/*  Desc:   0 => open file 1 => writing 2 => close file                         */
/*==============================================================================*/
#define BusOutModeSet(mode) BusoutMode = mode

/*==============================================================================*/
/*  Func:   _TP_BusOut                                                          */
/*  Desc:   Called by Test Point                                                */
/*==============================================================================*/
void _TP_BusOut(DT_UINT addr, DT_UINT dat)
{
	unsigned char buff[18] = {0};
	struct timespec now;

#if DT_ADD_EVENT_INFO
	pid = getpid();
#endif
	if (BusoutMode == 0) {
		/* Customize File Path */
		dat_file = fopen("ConvertData.DAT", "wb+");
		if (dat_file == NULL) {
			return;
		}
		/* semapho */
#if DT_USE_CRITICALSECTION
		SEM_INIT;
#endif
		BusOutModeSet(1);
	}

#if DT_USE_CRITICALSECTION
		SEM_WAIT;
#endif

	/* time data */
	if (clock_gettime(CLOCK_REALTIME, &now) == -1) {
		printf("DT10 : timer error!\n");
	}

	buff[0] = (unsigned char)((dat) & 0xFF);
	buff[1] = (unsigned char)((dat >> 8) & 0xFF);
	buff[2] = (unsigned char)((addr) & 0xFF);
	buff[3] = (unsigned char)((addr >> 8) & 0xFF);
	buff[4] = (unsigned char)((addr >> 16) & 0xFF);
	/* time data */
	buff[5] = (unsigned char)((now.tv_nsec) & 0xFF);
	buff[6] = (unsigned char)((now.tv_nsec >> 8) & 0xFF);
	buff[7] = (unsigned char)((now.tv_nsec >> 16) & 0xFF);
	buff[8] = (unsigned char)((now.tv_nsec >> 24) & 0xFF);
	buff[9] = (unsigned char)((now.tv_sec) & 0xFF);
	buff[10] = (unsigned char)((now.tv_sec >> 8) & 0xFF);
	buff[11] = (unsigned char)((now.tv_sec >> 16) & 0xFF);
	buff[12] = (unsigned char)((now.tv_sec >> 24) & 0xFF);

	buff[13] = 0x00;

#if DT_ADD_EVENT_INFO
	/* pid */
	buff[14] = (unsigned char)((pid) & 0xFF);
	buff[15] = (unsigned char)((pid >> 8) & 0xFF);
	buff[16] = (unsigned char)((pid >> 16) & 0xFF);
	buff[17] = (unsigned char)((pid >> 24) & 0xFF);
#else
	buff[14] = 0x00;
	buff[15] = 0x00;
	buff[16] = 0x00;
	buff[17] = 0x00;
#endif

	fwrite(buff, sizeof(char), 18, dat_file);
	if (BusoutMode == 2) {
		fclose(dat_file);
	}
#if DT_USE_CRITICALSECTION
	SEM_POST;
#endif
}

/*==============================================================================*/
/*  Func:   _TP_MemoryOutput                                                    */
/*  Desc:   Called by Variable TestPoint                                        */
/*==============================================================================*/
void _TP_MemoryOutput(DT_UINT addr, DT_UINT dat, void *value, DT_UINT size)
{
	char buff[18];
	char *p = value;
	int i, j, packetNum, count;

	struct timespec now;

#if DT_ADD_EVENT_INFO
	pid = getpid();
#endif
	if (BusoutMode == 0) {
		/* Customize File Path */
		dat_file = fopen("ConvertData.DAT", "wb+");
		if (dat_file == NULL) {
			return;
		}
		/* semapho */
#if DT_USE_CRITICALSECTION
		SEM_INIT;
#endif
		BusOutModeSet(1);
	}

#if DT_USE_CRITICALSECTION
		SEM_WAIT;
#endif
	if (size > 256) size = 256;
	packetNum = (size + 7) / 8;

	addr = addr | DT_VARIABLE_BIT;	/* Memory Dump Start */

	/* time data */
	if (clock_gettime(CLOCK_REALTIME, &now) == -1) {
		printf("DT10 : timer error!\n");
	}

	buff[0] = (unsigned char)((dat) & 0xFF);
	buff[1] = (unsigned char)((dat >> 8) & 0xFF);
	buff[2] = (unsigned char)((addr) & 0xFF);
	buff[3] = (unsigned char)((addr >> 8) & 0xFF);
	buff[4] = (unsigned char)((addr >> 16) & 0xFF);
	/* time data */
	buff[5] = (unsigned char)((now.tv_nsec) & 0xFF);
	buff[6] = (unsigned char)((now.tv_nsec >> 8) & 0xFF);
	buff[7] = (unsigned char)((now.tv_nsec >> 16) & 0xFF);
	buff[8] = (unsigned char)((now.tv_nsec >> 24) & 0xFF);
	buff[9] = (unsigned char)((now.tv_sec) & 0xFF);
	buff[10] = (unsigned char)((now.tv_sec >> 8) & 0xFF);
	buff[11] = (unsigned char)((now.tv_sec >> 16) & 0xFF);
	buff[12] = (unsigned char)((now.tv_sec >> 24) & 0xFF);

	buff[13] = 0x00;

#if DT_ADD_EVENT_INFO
	/* pid */
	buff[14] = (unsigned char)((pid) & 0xFF);
	buff[15] = (unsigned char)((pid >> 8) & 0xFF);
	buff[16] = (unsigned char)((pid >> 16) & 0xFF);
	buff[17] = (unsigned char)((pid >> 24) & 0xFF);
#else
	buff[14] = 0x00;
	buff[15] = 0x00;
	buff[16] = 0x00;
	buff[17] = 0x00;
#endif
	fwrite(buff, sizeof(char), 18, dat_file);

	buff[0] = size;
	buff[1] = packetNum;
	buff[2] = 0x03;
	buff[12] = 0x00;
	buff[13] = 0x00;
	buff[14] = 0x00;
	buff[15] = 0x00;
	buff[16] = 0x00;
	buff[17] = 0x00;

	count = 0;
	for (i = 0; i < packetNum; i++) {
		buff[3] = i;
		for (j = 0; j < 8; j++, count++, p++) {
			buff[4+j] = (count < size)?*p:0x00;
		}
		fwrite(buff, sizeof(char), 18, dat_file);
	}
	if (BusoutMode == 2) {
		fclose(dat_file);
	}
#if DT_USE_CRITICALSECTION
	SEM_POST;
#endif

}
